@@ -129,16 +129,15 @@ def specs_from_args(args, json=False):
     return [arg2spec(arg, json=json) for arg in args]
 
 
-def to_txn(specs_to_add, specs_to_remove, prefix, to_link, to_unlink, index=None):
+def to_txn(specs_to_add, specs_to_remove, prefix, to_link, to_unlink, index=[]):
     to_link_records, to_unlink_records = [], []
 
     prefix_data = PrefixData(prefix)
     final_precs = IndexedSet(prefix_data.iter_records())
 
-    def get_channel(c):
-        for _, chan in index:
-            if str(chan) == c:
-                return chan
+    lookup_dict = {}
+    for _, c in index:
+        lookup_dict[str(c)] = c
 
     for c, pkg in to_unlink:
         for i_rec in installed_pkg_recs:
@@ -150,7 +149,7 @@ def to_txn(specs_to_add, specs_to_remove, prefix, to_link, to_unlink, index=None
             print("No package record found!")
 
     for c, pkg, jsn_s in to_link:
-        sdir = get_channel(c)
+        sdir = lookup_dict[c]
         rec = to_package_record_from_subjson(sdir, pkg, jsn_s)
         final_precs.add(rec)
         to_link_records.append(rec)
@@ -368,13 +367,9 @@ def install(args, parser, command='install'):
 
     prefix = context.target_prefix
 
-    index_args = {
-        'use_cache': args.use_index_cache,
-        'channel_urls': context.channels,
-        'unknown': args.unknown,
-        'prepend': not args.override_channels,
-        'use_local': args.use_local
-    }
+    #############################
+    # Get SPECS                 #
+    #############################
 
     args_packages = [s.strip('"\'') for s in args.packages]
     if newenv and not args.no_default_packages:
@@ -395,42 +390,6 @@ def install(args, parser, command='install'):
             raise CondaValueError("cannot mix specifications with conda package"
                                   " filenames")
 
-    index = get_index(channel_urls=index_args['channel_urls'],
-                      prepend=index_args['prepend'], platform=None,
-                      use_local=index_args['use_local'], use_cache=index_args['use_cache'],
-                      unknown=index_args['unknown'], prefix=prefix)
-
-    channel_json = []
-    strict_priority = (context.channel_priority == ChannelPriority.STRICT)
-
-    if strict_priority:
-        # first, count unique channels
-        n_channels = len(set([channel.canonical_name for _, channel in index]))
-        current_channel = index[0][1].canonical_name
-        channel_prio = n_channels
-
-    for subdir, chan in index:
-        # add priority here
-        if strict_priority:
-            if chan.canonical_name != current_channel:
-                channel_prio -= 1
-                current_channel = chan.canonical_name
-            priority = channel_prio
-        else:
-            priority = 0
-
-        subpriority = 0 if chan.platform == 'noarch' else 1
-
-        if subdir.loaded() == False and chan.platform != 'noarch':
-            # ignore non-loaded subdir if channel is != noarch
-            continue
-
-        if context.verbosity != 0:
-            print("Cache path: ", subdir.cache_path())
-        channel_json.append((chan, subdir.cache_path(), priority, subpriority))
-
-    installed_json_f = get_installed_jsonfile(prefix)
-
     specs = []
 
     if args.file:
@@ -448,6 +407,65 @@ def install(args, parser, command='install'):
 
     specs.extend(specs_from_args(args_packages, json=context.json))
 
+    channels = [c for c in context.channels]
+    for spec in specs:
+        # CONDA TODO: correct handling for subdir isn't yet done
+        spec_channel = spec.get_exact_value('channel')
+        if spec_channel and spec_channel not in channels:
+            channels.append(spec_channel)
+
+    index_args = {
+        'use_cache': args.use_index_cache,
+        'channel_urls': channels,
+        'unknown': args.unknown,
+        'prepend': not args.override_channels,
+        'use_local': args.use_local
+    }
+
+    index = get_index(channel_urls=index_args['channel_urls'],
+                      prepend=index_args['prepend'], platform=None,
+                      use_local=index_args['use_local'], use_cache=index_args['use_cache'],
+                      unknown=index_args['unknown'], prefix=prefix)
+
+    channel_json = []
+    strict_priority = (context.channel_priority == ChannelPriority.STRICT)
+    subprio_index = len(index)
+    if strict_priority:
+        # first, count unique channels
+        n_channels = len(set([channel.canonical_name for _, channel in index]))
+        current_channel = index[0][1].canonical_name
+        channel_prio = n_channels
+
+    for subdir, chan in index:
+        # add priority here
+        if strict_priority:
+            if chan.canonical_name != current_channel:
+                channel_prio -= 1
+                current_channel = chan.canonical_name
+            priority = channel_prio
+        else:
+            priority = 0
+        if strict_priority:
+            subpriority = 0 if chan.platform == 'noarch' else 1
+        else:
+            subpriority = subprio_index
+            subprio_index -= 1
+
+        if subdir.loaded() == False and chan.platform != 'noarch':
+            # ignore non-loaded subdir if channel is != noarch
+            continue
+
+        if context.verbosity != 0:
+            print("Channel: {}, prio: {} : {}".format(chan, priority, subpriority))
+            print("Cache path: ", subdir.cache_path())
+
+        channel_json.append((chan, subdir, priority, subpriority))
+
+    # for c in channel_json:
+    #     print(c[0], c[2], c[3])
+
+    installed_json_f = get_installed_jsonfile(prefix)
+
     if isinstall and args.revision:
         get_revision(args.revision, json=context.json)
     elif isinstall and not (args.file or args_packages):
@@ -488,16 +506,15 @@ def install(args, parser, command='install'):
 
     # If python was not specified, check if it is installed.
     # If yes, add the installed python to the specs to prevent updating it.
-    python_added = False
+    python_constraint = None
     if 'python' not in spec_names:
         installed_names = [i_rec.name for i_rec in installed_pkg_recs]
         if 'python' in installed_names:
             i = installed_names.index('python')
             version = installed_pkg_recs[i].version
-            specs.append(MatchSpec('python==' + version))
-            python_added = True
+            python_constraint = MatchSpec('python==' + version).conda_build_form()
 
-    mamba_solve_specs = [s.conda_build_form() for s in specs]
+    mamba_solve_specs = [s.__str__() for s in specs]
 
     pool = api.Pool()
 
@@ -514,13 +531,17 @@ def install(args, parser, command='install'):
         repo.set_installed()
         repos.append(repo)
 
-    for channel, cache_file, priority, subpriority in channel_json:
-        repo = api.Repo(pool, str(channel), cache_file, channel.url(with_credentials=True))
+    for channel, subdir, priority, subpriority in channel_json:
+        repo = subdir.create_repo(pool)
         repo.set_priority(priority, subpriority)
         repos.append(repo)
 
     solver = api.Solver(pool, solver_options)
     solver.add_jobs(mamba_solve_specs, solver_task)
+
+    if python_constraint:
+        solver.add_constraint(python_constraint)
+
     solver.set_postsolve_flags(
         [(api.MAMBA_NO_DEPS, context.deps_modifier == DepsModifier.NO_DEPS), 
          (api.MAMBA_ONLY_DEPS, context.deps_modifier == DepsModifier.ONLY_DEPS)]
@@ -545,12 +566,10 @@ def install(args, parser, command='install'):
         exit(0)
     PackageCacheData.first_writable().reload()
 
-    if python_added:
-        specs_to_add = [s for s in specs_to_add if s.name != 'python']
-
     if use_mamba_experimental and not os.name == 'nt':
-        if command == 'create' and not isdir(context.target_prefix) and not context.dry_run:
+        if newenv and not isdir(context.target_prefix) and not context.dry_run:
             mkdir_p(prefix)
+
         transaction.execute(prefix_data, PackageCacheData.first_writable().pkgs_dir)
     else:
         conda_transaction = to_txn(specs_to_add, specs_to_remove, prefix, to_link, to_unlink, index)
@@ -617,6 +636,8 @@ def repoquery(args, parser):
     only_installed = True
     if args.subcmd == "search" and args.installed == False:
         only_installed = False
+    elif args.all_channels:
+        only_installed = False
 
     if not only_installed:
         index = get_index(channel_urls=index_args['channel_urls'],
@@ -633,16 +654,16 @@ def repoquery(args, parser):
             repo.set_priority(0, 0)
             repos.append(repo)
 
-
-    print("\nExecuting the query %s\n" % args.package_query)
+    if not context.json:
+        print("\nExecuting the query %s\n" % args.package_query)
 
     query = api.Query(pool)
     if args.subcmd == "whoneeds":
-        print(query.whatrequires(args.package_query, args.tree))
+        query.whoneeds(args.package_query, args.tree)
     if args.subcmd == "depends":
-        print(query.dependencytree(args.package_query))
+        query.depends(args.package_query)
     if args.subcmd == "search":
-        print(query.find(args.package_query))
+        query.find(args.package_query)
 
 
 def do_call(args, parser):
@@ -700,6 +721,12 @@ Examples:
         help=SUPPRESS
     )
 
+    package_cmds.add_argument(
+        "-a", "--all-channels",
+        action="store_true",
+        help="Look at all channels (for depends / whoneeds)"
+    )
+
     view_cmds = argparse.ArgumentParser(add_help=False)
     view_cmds.add_argument(
         "-t", "--tree",
@@ -725,6 +752,7 @@ Examples:
     conda_argparse.add_parser_channels(p)
     conda_argparse.add_parser_networking(p)
     conda_argparse.add_parser_known(p)
+    conda_argparse.add_parser_json(p)
 
     p.set_defaults(func='.main_repoquery.execute')
     return p
